<?php
if (!defined('MEDIAWIKI')) die("MediaWiki extensions cannot be run directly.");
/**
 * WikiDB
 * A MediaWiki extension which adds database functionality to your wiki, which
 * behaves in a truly wiki-like way.
 *
 * The extension is fairly well developed, but there are a lot more features
 * in the pipeline.  Visit my test wiki for documentation and updates, to get help
 * or to leave feedback.  See $pWikiDB_URL (below) for the URL.  Bug reports, 
 * patches and feature suggestions welcome.
 *
 * @package MediaWiki
 * @subpackage Extensions
 *
 * @author Mark Clements <mclements at kennel17 dot co dot uk>
 * @copyright Copyright  2006-2012, Mark Clements
 * @license http://creativecommons.org/licenses/by-sa/2.5/ cc-by-sa 2.5 or later
 * @version $Rev: 667 $
 */

//////////////////////////
// Credit information

// The version is an integer, and is incremented manually every time changes are
// made that require command-line update scripts to be run.  This includes both 
// changes to the DB table schema and changes that might invalidate existing WikiDB
// data, requiring a rebuild.
// Changes that will not break existing installations of WikiDB do not cause 
// this number to be updated.
	$pWikiDB_Version = 3;

// The revision is also an integer, and will hold the highest revision number of
// any WikiDB files.  Each file needs to call WikiDB_RegisterRevision() with its
// SVN revision string, and the highest number registered in this way will be
// used as the revision number for the extension.
// This is held in $pWikiDB_Revision, which is initialised to 0.
// This does not need any manual intervention so long as I remember to add a call to
// WikiDB_RegisterRevision() at the top of any new files, and set the svn:keywords
// property correctly.
// Note that changes to files in the maintenance directory do not cause the
// revision number to update.  Not sure if I can reasonably do anything about that!
	$pWikiDB_Revision = 0;
	WikiDB_RegisterRevision('$Rev: 667 $');

// Set these variables here so they can be re-used for each registered item.
// Needs to be done before SpecialPages.php is included, as they are used there
// too.
	$pWikiDB_Author = "Mark Clements";
	$pWikiDB_URL = "http://www.kennel17.co.uk/testwiki/WikiDB";


//////////////////////////
// Include files

// Need to ensure that the IncludableSpecialPage class is defined before we define
// our own special pages, so include the appropriate MW class file (if not already
// included).
	require_once("SpecialPage.php");

// Include all the other WikiDB files:
// Need to define WikiDB to let them know that it is safe to load.
	define("WikiDB", true);

// We calculate the include path here, so it only needs to be done once.
	$pWikiDB_IncludePath = dirname(__FILE__) . "/includes/";

// Support files
	require_once($pWikiDB_IncludePath . "Constants.php");
	require_once($pWikiDB_IncludePath . "Utility.php");
	require_once($pWikiDB_IncludePath . "Regex.php");
	require_once($pWikiDB_IncludePath . "BuiltInTypes.php");

// Core WikiDB classes
	require_once($pWikiDB_IncludePath . "classWikiDB.php");
	require_once($pWikiDB_IncludePath . "classWikiDB_Table.php");
	require_once($pWikiDB_IncludePath . "classWikiDB_Query.php");
	require_once($pWikiDB_IncludePath . "classWikiDB_Parser.php");
	require_once($pWikiDB_IncludePath . "classWikiDB_TypeHandler.php");

// MW interface hooks
	require_once($pWikiDB_IncludePath . "Hooks.php");
	require_once($pWikiDB_IncludePath . "ParserTags.php");
	require_once($pWikiDB_IncludePath . "SpecialPages.php");
//	require_once($pWikiDB_IncludePath . "MagicWords.php");

	unset($pWikiDB_IncludePath);

//////////////////////////
// Extension credits:
// Note that special page credits are registered in includes/SpecialPages.php.

// Main extension credits
	$wgExtensionCredits['other'][] = array(
		'name' => "WikiDB",
		'author' => $pWikiDB_Author,
		'description' => "a wiki-based database system",
		'version' => $pWikiDB_Version . ", r" . $pWikiDB_Revision,
		'url' => $pWikiDB_URL,
	);


//////////////////////////
// Variable initialisation

// If $wgWikiDBNamespaces has not been set, initialise it to an empty array.
	if (!isset($wgWikiDBNamespaces))
		$wgWikiDBNamespaces = array();

// If $wgWikiDBMaxRefreshRate has not been set, initialise it to the default value.
// This variable can be either set to a number indicating the number of 
// stale rows to refresh on each page view, or to one of the following constants:
// 	WIKIDB_DisableAutoRefresh
//		Disables this auto-refresh completely (presumably because you are handling 
//		it via a cron job).
// 	WIKIDB_RefreshAll
//		Update all stale rows on every page request (if there are any), so there 
//		is never any stale data.
// The optimum value for this setting will depend on how many visitors you get, 
// how large your tables are (in terms of numbers of rows) and how often your table
// definitions change.  The default has been chosen as a good average-case
// value, but you will probably want to customise this.
// Note that the recommended method of refreshing stale links is to run 
// RefreshStaleData.php via a regular cron job, in which case you should set this
// variable to WIKIDB_DisableAutoRefresh.
	if (IsBlank(@$wgWikiDBMaxRefreshRate))
		$wgWikiDBMaxRefreshRate = pWIKIDB_DefaultMaxRefreshRate;

// Private variable so that our parser can indicate that it is parsing for the
// purposes of saving data.  This causes the functions that handle the <data> tag 
// and table definitions to store the data to the DB, rather than rendering it for 
// output, and causes the other tags (which only affect rendering) to be skipped 
// completely.
	$wgpWikiDBParsingForSave = false;

//////////////////////////
// Extension registration

	$wgExtensionFunctions[] = "wfWikiDB";

// It seems that in MW < 1.18, we can't load our message file inside our startup
// function - perhaps it is too late in the process?  Therefore we set it up here.
	global $pWikiDB_MessagesFile;
	$pWikiDB_MessagesFile = dirname( __FILE__ ) . '/WikiDB.i18n.php';

	if (isset($wgExtensionMessagesFiles))
		$wgExtensionMessagesFiles['WikiDB'] = $pWikiDB_MessagesFile;
	
	function wfWikiDB() {
		global $wgMessageCache, $wgHooks, $wgParser, $wgVersion, $wgCommandLineMode;
		global $pWikiDB_Messages, $wgWikiDBMaxRefreshRate, $pWikiDB_MessagesFile;
		global $wgExtensionMessagesFiles;

	// Localisable messages
		if (!isset($wgExtensionMessagesFiles)) {
			require_once($pWikiDB_MessagesFile);
			$wgMessageCache->addMessages($messages['en']);
		}

	// Application hooks
		$wgHooks['SkinTemplateContentActions'][] = "wfWikiDB_AddActionTabs";
		$wgHooks['SkinTemplateSetupPageCss'][] = "wfWikiDB_AddCSS";
		$wgHooks['UnknownAction'][] = "wfWikiDB_SpecialActions";
		$wgHooks['ParserBeforeStrip'][] = "wfWikiDB_ParserBeforeStrip";
		$wgHooks['ArticleSaveComplete'][] = "wfWikiDB_ArticleSaveComplete";
		$wgHooks['ArticleDeleteComplete'][] = "wfWikiDB_ArticleDeleteComplete";
		$wgHooks['TitleMoveComplete'][] = 'wfWikiDB_TitleMoveComplete';
	
	// MW 1.18 replaced some of the SkinTemplate* hooks with new hooks that have
	// been generalised, and which are designed around the new Vector skin.
	// We therefore need to set up an additional callback in order to ensure that
	// the 'viewdata' tab is present in 1.18.  In earlier versions, the
	// 'SkinTemplateContentActions' will be used.  In 1.18 and above, this hook no
	// longer exists, and 'SkinTemplateNavigation::Universal' is used instead.
	// As the old hook was removed at the same time as the new one was added, we
	// don't need to do a version check to avoid the hook being called twice.
		$wgHooks['SkinTemplateNavigation::Universal'][] = "wfWikiDB_SkinHooks";

	// If MW version < 1.9.1, the ArticleUndelete hook wasn't present, so we need to
	// use a bit of a cheeky workaround...
		if (version_compare($wgVersion, '1.9.1', '<')) {
			$wgHooks['SpecialPageExecuteAfterPage'][] =
									'wfWikiDB_SpecialPageExecuteAfterPage';
		}
	// Otherwise we use ArticleUndelete
		else {
			$wgHooks['ArticleUndelete'][] = 'wfWikiDB_ArticleUndelete';
		}

	// Parser hooks
	     $wgParser->setHook("data", "wfWikiDB_DataTag");
	     $wgParser->setHook("repeat", "wfWikiDB_RepeatTag");
	     $wgParser->setHook("guesstypes", "wfWikiDB_GuessTypes");

	// Special pages
		SpecialPage::addPage(new WikiDB_SP_AllTables());
		SpecialPage::addPage(new WikiDB_SP_UndefinedTables());
		SpecialPage::addPage(new WikiDB_SP_EmptyTables());
	
	// On each page request, we update any stale data, up to the limit imposed
	// by $wgWikiDBMaxRefreshRate.  If that variable is set to the constant
	// WIKIDB_DisableAutoRefresh then auto-refresh is skipped (presumably because
	// a regular cron-job is handling it instead).
	// We also skip this in command-line mode, as there is a dedicated script for 
	// this and we don't want to intefere with the running of that or any other
	// scripts.
	// In the most common case, where there is no stale data, the speed implication 
	// should be negligible - if that is not the case then this will need revisiting.
		if (!$wgCommandLineMode
			&& $wgWikiDBMaxRefreshRate !== WIKIDB_DisableAutoRefresh)
		{
			WikiDB::RefreshStaleFieldData();
		}
	}


//////////////////////////
// Other registration functions

	function WikiDB_RegisterRevision($RevString) {
		global $pWikiDB_Revision;

		if (preg_match('/\$.*:\s?(\d+)\s?\$/', $RevString, $Matches)) {
			$Revision = intval($Matches[1]);
			if ($Revision > $pWikiDB_Revision)
				$pWikiDB_Revision = $Revision;
		}
	}
