<?php
if (!defined('MEDIAWIKI')) die("MediaWiki extensions cannot be run directly.");
if (!defined('WikiDB')) {
	die("The WikiDB extension has not been set up correctly.<br>'"
		. basename(__FILE__) . "' should not be included directly.<br>Instead, "
		. "add the following to LocalSettings.php: "
		. "<tt><b>include_once(\"WikiDB/WikiDB.php\");</b></tt>");
}
/**
 * General-purpose utility functions for WikiDB.
 * These may be useful in other applications too.
 *
 * To avoid conflicts with other extensions (particularly my own) I either check
 * that the function hasn't already been defined prior to defining it, or else
 * I've prefixed the function name with "WikiDB_", to make the name unique.
 *
 * @package MediaWiki
 * @subpackage Extensions
 *
 * @author Mark Clements <mclements at kennel17 dot co dot uk>
 * @copyright Copyright  2006-2012, Mark Clements
 * @license http://creativecommons.org/licenses/by-sa/2.5/ cc-by-sa 2.5 or later
 * @version $Rev: 661 $
 */

WikiDB_RegisterRevision('$Rev: 661 $');

// Converts a string to a number.  Either an int or a float as appropriate.
// Zero is returned for non-numeric strings.
	function WikiDB_StrToNumber($Value) {
		if (is_numeric($Value)) {
			if (strpos($Value, ".") !== false)
				return floatval($Value);
			else
				return intval($Value);
		}
		else
			return 0;
	}

// Formats a float value as a string, with trailing zeros removed and a minimum of
// 1 digit before the decimal point.
	function WikiDB_FormatFloat($Value) {
		$Value = sprintf("%1f", $Value);
		return preg_replace("/(\.\d+)0+$/U", "$1", $Value);
	}

// TODO - this is VERY crude at the moment, and may well fail in certain edge cases
// (or even, certain 'non-edge' cases!)
	function pWikiDB_ExpandVariables($Input, $Data) {
		if (is_array($Data) && count($Data) > 0) {
		// Substitute variables.  Note that we need to replace $ with \$ within
		// $Value, to stop it being interpreted as a PHP variable reference.
			foreach ($Data as $Key => $Value) {
				$Input = preg_replace("/{{{" . preg_quote($Key, "/") . "}}}/",
									  str_replace('$', '\$', $Value),
									  $Input);
			}
		}
		return $Input;
	}

// Wrapper function to simplify text parsing.
// We use a local instance of the parser class, so that we don't get into a
// recursive nightmare (e.g. if a <repeat> tag contains a <repeat> tag).
// All tag hooks are ignored (as they are being dealt with already), however
// parser function hooks are parsed as expected.
	function WikiDB_Parse($Input, &$Parser, $LineStart = true) {
		global $wgParser;
		static $LocalParser;
		
		if (!isset($LocalParser)) {
			$LocalParser = new Parser;
		
		// Later versions of MW (tested on 1.16) automatically add the necessary parser 
		// function hooks to a new Parser instance, however earlier versions (tested
		// on 1.6) don't.  If they are not present, copy them from the global
		// $wgParser object.
			if (count($LocalParser->mFunctionHooks) == 0)
				$LocalParser->mFunctionHooks = $wgParser->mFunctionHooks;
		}

		$Output = $LocalParser->parse($Input, $Parser->mTitle, $Parser->mOptions,
									  $LineStart, true);
		return $Output->getText();
	}

// WikiDB_MakeTitle()
// $Title may either be a title object or a string representing the name of a
// page.  If $Title is a string, but doesn't include a namespace, then
// $DefaultNamespace is used if specified.  If no default is specified then 
// WikiDB::GetDefaultTableNamespace() is used to fill in a sensible default.
	function WikiDB_MakeTitle($Title, $DefaultNamespace = NULL) {
	// If $Title is already a Title object, just return it.
	// TODO: Currently we assume that if it is an object, it is a Title object.
	//		 Is this sensible?
		if (is_object($Title)) {
			return $Title;
		}
	// Otherwise, it is a text string representing a page title.
	// In this case we create a new title object from that string, using the
	// default table namespace if necessary.
		else {
		// If no default namespace is specified, use the standard default.
			if ($DefaultNamespace === NULL)
				$DefaultNamespace = WikiDB::GetDefaultTableNamespace();

		// TODO: Should this use Title::newFromDBKey() instead?  If $Title contains
		//		 text it *should* always be a DB key, but it seems that the
		//		 transformations involved in newFromText() won't affect strings that
		//		 are already transformed, therefore it is safe (if marginally slower)
		//		 to pass it through there anyway - and that avoids problems if we
		//		 get passed in the actual title, rather than the DB key, for any
		//		 reason.
			return Title::newFromText($Title, $DefaultNamespace);
		}
	}

// GetCanonicalNamespaceName()
// Returns the canonical name of the specified namespace.  I've written a wrapper
// function, as this isn't as simple as I'd like!
	function WikiDB_GetCanonicalNamespaceName($NS) {
		global $wgCanonicalNamespaceNames;

	// If the namespace exists in $wgCanonicalNamespaceNames then get its canoncial
	// name using the getCanonicalName() function of the MWNamespace class.
	// Originally, this class was called Namespace, but this became a reserved word in
	// PHP 5.3 and so in MW 1.13 it was renamed to MWNamespace.  Namespace remained
	// as an alias until it was finally removed altogether in MW 1.17.
		if (isset($wgCanonicalNamespaceNames[$NS])) {
		// In MW >= 1.13, the class we need is called MWNamespace, so if the MWNamespace
		// class exists, use it.
			if (class_exists("MWNamespace")) {
				$NSName = MWNamespace::getCanonicalName($NS);
			}
		// If MWNamespace is unavailable, then we are using a verions of MW < 1.13.
		// In these earlier versions, the class was called Namespace, so we use that
		// class instead.  However, we can't simply call this class in the normal
		// fashion, as it would cause parse errors if running on PHP >= 5.3, therefore 
		// we need to use eval() to run the necessary code, as this won't be parsed
		// until it is actually executed.
			else {
				$PHPCode = 'return Namespace::getCanonicalName(' . intval($NS) . ');';
				$NSName = eval($PHPCode);
			}
		}
	// If it is not defined, but $NS = 0, then it is the main namespace.  This doesn't
	// have an actual name, but we still need to report it.  We therefore use 'main' for
	// this.
	// TODO: Can this be internationalised?
		elseif ($NS == 0) {
			$NSName = "main";
		}
	// Otherwise, it is an undefined namespace.
	// TODO: Can this be internationalised?
		else {
			$NSName = "undefined";
		}
		
		return $NSName;
	}

if (!function_exists("IsBlank")) {
	function IsBlank($Input) {
	// is_null() returns true if $Input is unset or null
		if (is_null($Input))
			return true;
		elseif ($Input === "")
			return true;
		elseif (is_array($Input) && count($Input) == 0)
			return true; 
		else
			return false;
	}
}
