<?php
if (!defined('MEDIAWIKI')) die("MediaWiki extensions cannot be run directly.");
if (!defined('WikiDB')) {
	die("The WikiDB extension has not been set up correctly.<br>'"
		. basename(__FILE__) . "' should not be included directly.<br>Instead, "
		. "add the following to LocalSettings.php: "
		. "<tt><b>include_once(\"WikiDB/WikiDB.php\");</b></tt>");
}
/**
 * WikiDB class.
 * A set of static functions relating to the WikiDB database.
 *
 * @package MediaWiki
 * @subpackage Extensions
 *
 * @author Mark Clements <mclements at kennel17 dot co dot uk>
 * @copyright Copyright  2006-2012, Mark Clements
 * @license http://creativecommons.org/licenses/by-sa/2.5/ cc-by-sa 2.5 or later
 * @version $Rev: 665 $
 */

WikiDB_RegisterRevision('$Rev: 665 $');

class WikiDB {

	function WikiDB() {
	// Never instantiate this object!
	}
	
// GetDefaultTableNamespace()
// Returns the namespace ID of the default table namespace.
// This is the first namespace defined in $wgWikiDBNamespaces.  If no table namespace
// is defined then NS_MAIN is used.
// TODO: Falling back on NS_MAIN is probably a bad idea for many reasons!
	function GetDefaultTableNamespace() {
		global $wgWikiDBNamespaces;

	// We use the $Enabled variable to hold the value in our foreach loop.
	// We set it to false intially (so that if no table namespaces are defined
	// we don't get an undefined variable error) and it will be set to true if an
	// enabled table namespace is found.
		$Enabled = false;

	// If no default namespace specified, pick the first defined DB namespace.
	// This loops through the array of table namespaces setting the
	// $DefaultNamespace and $Enabled flag for each iteration.  If $Enabled is true
	// then we break out of the loop, as $DefaultNamespace has now been correctly
	// filled in.  If we get to the end of the loop $Enabled will still be false
	// so the value in $DefaultNamespace, which will be the last namespace in the
	// array, will subsequently be overwritten by the correct default.
		if (count($wgWikiDBNamespaces) > 0) {
			foreach ($wgWikiDBNamespaces as $DefaultNamespace => $Enabled) {
				if ($Enabled)
					break;
			}
		}

	// If no namespaces are defined, or there are defined namespaces but they
	// are all disabled, fall back on the Main namespace.
	// TODO: Is this wise? (see above)
		if (!$Enabled)
			$DefaultNamespace = NS_MAIN;

		return $DefaultNamespace;
	}

// pSetupPageObjects()
// Helper function, which takes a $PageObject, which may be a MediaWiki Title or
// Article object, a new WikiPage object (as of MW 1.18), or a WikiDB_Table object,
// and sets up the $Title and $Article variables appropriately.
// The $Title and $Article variables should be initialised to NULL by the calling 
// function and passed into this function which will populate them (as they are
// passed by reference).
// Returns false on error (i.e. $PageObject is not one of the recognised classes).
	function pSetupPageObjects($PageObject, &$Title, &$Article) {
		
		switch (strtolower(get_class($PageObject))) {
			case "wikipage":
				$Title = $PageObject->getTitle();
				$Article = new Article($Title);
				break;
			case "article":
				$Article = $PageObject;
				$Title = $PageObject->getTitle();
				break;
			case "title":
				$Article = new Article($PageObject);
				$Title = $PageObject;
				break;
			case "wikidb_table":
				$Article = $PageObject->GetArticle();
				$Title = $PageObject->GetTitle();
				break;
			default:
			// TODO: How should we handle this error scenario?
				return false;
		}
		
		return true;
	}

// PageUpdated()
//
// Updates the WikiDB tables for the given $PageObject with the given Text 
// (presumably the text of the current revision).  $PageObject can be any of the
// object types recognised by pSetupPageObjects().
// The $Text argument may be omitted, in which case the current article text
// is used.
// The function extracts all data tags from the $Text and stores them in the 
// database, replacing the existing data stored for this article.  Ditto field 
// definitions if this is in the table namespace. This code doesn't handle 
// rendering - it just extracts and saves the data.  See wfWikiDB_ParserAfterStrip()
// for rendering code.
//
// It is assumed that the MediaWiki code is transaction safe, so if either the main
// article save or the extension's DB changes fail then the other is rolled back if
// necessary.
//
// TODO: Investigate transacations, and if not safe some security measures will be
//		 needed in this function.
	function PageUpdated($PageObject, $Text = NULL) {
		global $wgParser;
		global $wgpWikiDBParsingForSave;

		$fname = __CLASS__ . "::PageUpdated";
		wfProfileIn($fname);
		
	// Setup $Title and $Article from $PageObject.
		$Title = NULL;
		$Article = NULL;
		if (!WikiDB::pSetupPageObjects($PageObject, $Title, $Article))
			return false;

	// Pages in the MediaWiki namespace can't contain data or table definitions.
		if ($Title->getNamespace() == NS_MEDIAWIKI)
			return false;

	// If no $Text argument was supplied, use the current article text. 
		if ($Text === NULL)
			$Text = $Article->getContent();

	// Delete any existing data for this page.
		WikiDB::pDeleteDataForPage($Title);

	// Parse the article.  This will pass through our various standard parsing 
	// functions, specifically the ParserAfterStrip callback (for table definitions)
	// and the <data> tag (for data).  These functions check the value of
	// $wgpWikiDBParsingForSave and if true they perform the appropriate save
	// action to update the DB, whilst if false they render the page for display.
	// This ensures that the page is parsed identically for rendering as it is for
	// saving.
		$wgpWikiDBParsingForSave = true;
	
	// Not sure how necessary setting these options is - it's only worthwhile
	// it it helps speed up the parse.
		$ParserOptions = new ParserOptions();

	// setUseTex was removed in MW 1.16 (r61913), so we need to check it exists 
	// before calling it.
		if (is_callable(array($ParserOptions, "setUseTex")))
			$ParserOptions->setUseTeX(false);

		$ParserOptions->setUseDynamicDates(false);
		$ParserOptions->setInterwikiMagic(false);
		$ParserOptions->setNumberHeadings(false);
		$ParserOptions->setTidy(false);

		$wgParser->parse($Text, $Title, $ParserOptions, 
						 true, true, $Article->getRevIdFetched());

		$wgpWikiDBParsingForSave = false;

	// NOTE: Originally I called RefreshStaleFieldData() at the end of this
	//		 function, so that we refresh stale data up to the limit imposed by
	// 		 $wgWikiDBMaxRefreshRate as part of the initial update.  However,
	//		 it appears that MW does a redirect after a page update, meaning that
	//		 the call to RefreshStaleFieldData() that happens on each page 
	//		 request will automatically kick in, and we don't want to run this
	//		 twice.  I don't know whether MW always redirects in this manner, or
	//		 if this is down to server configuration, so I'm leaving this note
	//		 here as a reminder to investigate and make sure that we will always
	//		 get an initial update.

		wfProfileOut($fname);
		return true;
	}

// pAddTableDefinition()
// Updates the DB to the new WikiDB table definition for the specified page
// (as defined by $TableTitle which is a MediaWiki Title object), based on parsing 
// the content provided in $PageText.
// It is assumed that the title is in a valid table namespace - it is up to the
// calling function to check this if it may not be the case.
	function pAddTableDefinition($TableTitle, $PageText) {
		$fname = __CLASS__ . "::pAddTableDefinition";
		$DB =& wfGetDB(DB_MASTER);

	// Delete the old definition.
	// TODO: This should probably be done using an update query if the page
	// 		 exists, or an insert if it is a new page, to save on a DB call.
	//		 However if we need a SELECT to work this out then this method might
	//		 work out just as fast...
		$DB->delete(pWIKIDB_tblTables,
					array(
						'table_namespace'	=> $TableTitle->getNamespace(),
						'table_title'		=> $TableTitle->getDBKey(),
					), $fname);

	// Generate the field data from the page contents, using the parser in 
	// the WikiDB class.
		$FieldData = WikiDB_Parser::ParseTableDef($PageText, false);

	// If page is a redirect, fill in the redirect fields.
	// We only add redirects that point to other tables.  Redirects pointing
	// to a namespace that is not a table namespace are ignored.
		$Redirect = Title::newFromRedirect($PageText);
		if ($Redirect !== NULL && WikiDB_Table::IsValidTable($Redirect)) {
			$RedirectNS = $Redirect->getNamespace();
			$RedirectTitle = $Redirect->getDBKey();
		}
		else {
			$RedirectNS = NULL;
			$RedirectTitle = NULL;
		}

	// Insert the new definition.
		$DB->insert(pWIKIDB_tblTables,
					array(
						'table_namespace'		=> $TableTitle->getNamespace(),
						'table_title'			=> $TableTitle->getDBKey(),
						'table_def'				=> serialize($FieldData),
						'redirect_namespace'	=> $RedirectNS,
						'redirect_title'		=> $RedirectTitle,
					), $fname, 'IGNORE');

	// Mark all rows that might be affected by modifications to this table
	// as stale, i.e. rows belonging to this table or any of its aliases (though
	// not for tables this is an alias of - they are unaffected).
		$Table = new WikiDB_Table($TableTitle);
		$Aliases = $Table->GetTableAliases();
		
		$Where = "";
		foreach ($Aliases as $Table) {
			if (!IsBlank($Where))
				$Where .= " OR ";
			$Where .= "(table_namespace = " 
					. $DB->addQuotes($Table->GetNamespace())
					. " AND table_title = "
					. $DB->addQuotes($Table->GetDBKey())
					. ")";
		}
		$SQL = "UPDATE " . $DB->tableName(pWIKIDB_tblRows) . " SET is_stale = 1 " 
			 . "WHERE " . $Where . ";";
		$DB->query($SQL, $fname);
	}

// pDeleteDataForPage()
// Deletes all cached data for the given page - either the page has been deleted
// or we are wiping existing data before a reparse.
	function pDeleteDataForPage($Title) {
		$fname = __CLASS__ . "::pDeleteDataForPage";
		$DB =& wfGetDB(DB_MASTER);

	// Retrieve existing data row_ids for this article
		$DataRows = $DB->select(pWIKIDB_tblRows, array('row_id'),
								array('page_namespace' 	=> $Title->getNamespace(),
									  'page_title' 		=> $Title->getDBKey())
								);

		$ExistingRowIDs = array();
		while ($Row = $DB->fetchObject($DataRows)) {
			$ExistingRowIDs[] = $Row->row_id;
		}
		$DB->freeResult($DataRows);

	// If there were existing rows, delete the old data.
		if (count($ExistingRowIDs) > 0) {
			$DB->delete(pWIKIDB_tblFields, array('row_id' => $ExistingRowIDs),
						$fname);
			$DB->delete(pWIKIDB_tblRows, array('row_id' => $ExistingRowIDs), $fname);
		}
	}

// pAddDataDefinition()
// Updates the DB to hold the data definition whose contents are held in 
// $DataTagText.  The data was located on $SourcePage and destined for the 
// $DestinationTable.
	function pAddDataDefinition($SourcePage, $DataTagText, $Args) {
		$fname = __CLASS__ . "::pAddDataDefinition";
		$DB =& wfGetDB(DB_MASTER);

	// Ensure that $DestinationTable is a WikiDB_Table object.
		$DestinationTable = new WikiDB_Table($Args['table']);

	// Parse the tag's contents to get the data.
		$ParsedData = WikiDB_Parser::ParseDataTag(
														$DataTagText,
														$Args['fields'],
														$Args['separator']
													);

		foreach ($ParsedData as $RowData) {
		// If any value=data pairs were found, save this row.
			if (count($RowData) > 0) {
				$DB->insert(pWIKIDB_tblRows,
							array(
								'page_namespace'	=> $SourcePage->getNamespace(),
								'page_title'    	=> $SourcePage->getDBKey(),
								'table_namespace'	=> $DestinationTable->GetNamespace(),
								'table_title'		=> $DestinationTable->GetDBKey(),
								'parsed_data'		=> serialize($RowData)
							), $fname, 'IGNORE');
				$RowID = $DB->insertId();

				WikiDB::pAddFieldDefs($RowID, $DestinationTable, $RowData, $DB, true);
			}
		}
	}

// pAddFieldDefs()
// Updates all field definitions for the specified row, based on the current
// definitions for the given table.  The table is resolved to the final data
// destination so that fields can be formatted appropriately.
// The $SkipDelete argument can be used to bypass the delete query if you know that
// it is unnecessary (e.g. when a page is updated and all records for that page were
// deleted in one go).
	function pAddFieldDefs($RowID, $Table, $Fields, $DB, $SkipDelete = false) {
		$fname = __CLASS__ . "::pAddFieldDefs";

		if (!$SkipDelete)
			$DB->delete(pWIKIDB_tblFields, array('row_id' => $RowID), $fname);

	// If the table is an alias it needs to be resolved so that we get the 
	// destination into which the fields will be put.  We need to do this in order
	// to use the correct field definitions for formatting the field values.
		$Table = $Table->GetDataDestination();

		foreach ($Fields as $FieldName => $FieldValue) {
		// We format the field value appropriately for the field's type.  If the
		// field has no definition then the string is returned unmodified (though
		// in either case it is truncated, if necessary, to fit into a VARCHAR(255)).
			$FieldValue = $Table->FormatFieldForSorting($FieldName, $FieldValue);

		// Note that the field name we store in the DB is the actual field name
		// specified - i.e. we don't resolve aliases.  Because there may be stale
		// data, our SQL for querying the data needs to check all aliases of a field
		// otherwise data might be missed, therefore there is no advantage to 
		// resolving it here.  If we decide to increase speed at the expense of 
		// accuracy then we could make an assumption that all aliases are resolved 
		// correctly and store the alias here instead - stale records may not be
		// handled correctly in that situation, however.

		// Insert the data row for this field into the DB.
			$DB->insert(pWIKIDB_tblFields,
						array(
							'row_id'			=> $RowID,
							'field_name'		=> $FieldName,
							'field_value'		=> $FieldValue,
						), $fname, 'IGNORE');
		}
	}

// PageDeleted()
//
// Updates the WikiDB tables for the given $PageObject on the basis that the page
// has now been deleted.  $PageObject can be any of the object types recognised by
// pSetupPageObjects().
	function PageDeleted($PageObject) {
		$fname = __CLASS__ . "::PageDeleted";
		wfProfileIn($fname);
		$DB =& wfGetDB(DB_MASTER);
		
	// Setup $Title and $Article from $PageObject.
		$Title = NULL;
		$Article = NULL;
		if (!WikiDB::pSetupPageObjects($PageObject, $Title, $Article))
			return false;

	// If we are in a table namespace, we first need to delete the table data.
		if (WikiDB_Table::IsValidTable($Title)) {
			$DB->delete(pWIKIDB_tblTables,
						array(
							'table_namespace'	=> $Title->getNamespace(),
							'table_title'		=> $Title->getDBKey(),
						), $fname);
		}

	// For all pages we need to delete any row/field data.

	// If using MySQL >= 4.0 then we can use deleteJoin to delete the field data.
	// Otherwise (if MySQL < 4.0) then we just delete the row data and leave the
	// field data lying around.
	// TODO: Check if it is safe to leave field data lying around...
		if (version_compare($DB->getServerVersion(), "4.0", ">=")) {
			$DB->deleteJoin(pWIKIDB_tblFields, pWIKIDB_tblRows,
							$DB->tableName(pWIKIDB_tblRows) . ".row_id",
							$DB->tableName(pWIKIDB_tblFields) . ".row_id",
							array(
								'page_namespace'	=> $Title->getNamespace(),
								'page_title'		=> $Title->getDBKey(),
							), $fname);
		}

		$DB->delete(pWIKIDB_tblRows,
					array(
						'page_namespace'	=> $Title->getNamespace(),
						'page_title'		=> $Title->getDBKey(),
					), $fname);
	}

// RefreshStaleFieldData()
// Refreshes the field data for any WikiDB rows marked as 'stale'.  If $BatchSize
// is specified then the number of rows that will be refreshed is restricted
// to this number, otherwise the value set in $wgWikiDBMaxRefreshRate is used.
// A value of WIKIDB_RefreshAll (passed in, or in the variable) means refresh all 
// values.  We don't worry about WIKIDB_DisableAutoRefresh, as this is handled
// in WikiDB.php before we get to this function.
// $ReportProgress should only be set to true when running from the command line.
	function RefreshStaleFieldData($BatchSize = NULL, $ReportProgress = false) {
		global $wgWikiDBMaxRefreshRate;

		$fname = __CLASS__ . "::RefreshStaleFieldData";
		wfProfileIn($fname);
		$DB =& wfGetDB(DB_MASTER);

		if ($BatchSize === NULL)
			$BatchSize = $wgWikiDBMaxRefreshRate;

		$Options = array();
		if ($BatchSize != 0)
			$Options['LIMIT'] = $BatchSize;

		$Res = $DB->select(pWIKIDB_tblRows, "*", array("is_stale" => true), 
						   $fname, $Options);

		if ($ReportProgress) {
			print("Rows processed: 0");
			$LastReport = 0;
			$i = 0;
		}

		while ($Row = $DB->fetchObject($Res)) {
			$Table = new WikiDB_Table($Row->table_title, $Row->table_namespace);
			$Fields = unserialize($Row->parsed_data);

			WikiDB::pAddFieldDefs($Row->row_id, $Table, $Fields, $DB);
			$DB->update(pWIKIDB_tblRows, array("is_stale" => false), 
						array("row_id" => $Row->row_id), $fname);

			if ($ReportProgress && (++$i % pWIKIDB_ReportingInterval) == 0) {
				print_c_fixed($LastReport, $i);
				$LastReport = $i;
			}
		}

		if ($ReportProgress)
			print_c_fixed($LastReport, $i . "\n");
	}

} // END: class WikiDB
