<?php
/**
 * WikiDB: General functions to help with the various command-line scripts.
 *
 * @package MediaWiki
 * @subpackage Extensions
 *
 * @author Mark Clements <mclements at kennel17 dot co dot uk>
 * @copyright Copyright  2006-2012, Mark Clements
 * @license http://creativecommons.org/licenses/by-sa/2.5/ cc-by-sa 2.5 or later
 * @version $Rev: 666 $
 */

//////////////////////// SQL HELPERS

// Helper functions for installation code.
// This was written for WikiDB, but some bits may be useful for other
// extensions.

// ApplySQLFile()
// Applies the specified SQL file to the specified DB, outputting the message
// beforehand and an indication of success afterwards.
// TODO: Do we need to provide a separate postgre version of these SQL files?
//		 Use ($wgDBtype === 'postgres') as the test, if this is required, and pick
//		 the appropriate version from within this function.
function ApplySQLFile($Message, $SQLFileName, &$DB) {
	print($Message . "...");

	$Result = $DB->sourceFile(dirname( __FILE__ ) . "/sql/" . $SQLFileName);

	if (is_string($Result)) {
		print("ERROR!\n" . $Result . "\n");
		exit(1);
	}
	else
		print("ok\n");
}

function RemoveField($Table, $Field, &$DB) {
	if (!$DB->tableExists($Table)) {
		print("Warning: Attempt to remove " . $Field 
			  . " field from non-existant table, " . $Table . "!\n");
	}
	elseif (!$DB->fieldExists($Table, $Field)) {
		print("..." . $Field . " field already removed from " . $Table 
			  . " table.\n");
	}
	else {
		print("Removing " . $Field . " field from " . $Table . " table....");
		$SQL = "ALTER TABLE " . $DB->tableName($Table)
			 . " DROP COLUMN " . $Field . ";";
		$DB->query($SQL, "WikiDB installer: RemoveField()");
		print("ok\n");
	}
}

//////////////////////// WIKIDB HELPERS

// Based on fixLinksFromArticle()
// Reparses given article using current parser state.  You should disable
// any unnecessary tags/extensions/etc. before this is called, to improve speed.
function ReparseArticle($ArticleID) {
	global $wgTitle, $wgParser;
	$fname = "ReparseArticle";
	
	$wgTitle = Title::newFromID($ArticleID);
	$DB =& wfGetDB(DB_MASTER);
	
	if (is_null($wgTitle))
		return;

	$DB->begin();

	$Revision = Revision::newFromTitle($wgTitle);

	if ($Revision) {
	// We need this next line to stop problems with the parser referring to mTitle,
	// e.g. in areSubpagesAllowed().
		$wgParser->mTitle = $wgTitle;
		WikiDB::PageUpdated($wgTitle, $Revision->getText());
	}

	$DB->commit();
}

function ClearTableDefCache($NS = NULL) {
	$fname = "ClearTableDefCache";

	$DB =& wfGetDB(DB_WRITE);
	
	if ($NS === NULL) {
		$AffectedRows = $DB->selectField(pWIKIDB_tblTables, "COUNT(*)", "", $fname);
		$SQL = "TRUNCATE " . $DB->tableName(pWIKIDB_tblTables);
		$DB->query($SQL, $fname);
	}
	else {
		$SQL = "DELETE FROM " . $DB->tableName(pWIKIDB_tblTables) 
			 . " WHERE table_namespace = " . $DB->addQuotes($NS);

		$DB->query($SQL, $fname);
		$AffectedRows = $DB->affectedRows();
	}

	return $AffectedRows;
}

function ClearAllWikiDBCaches() {
	$fname = "ClearAllWikiDBCaches";

	$DB =& wfGetDB(DB_WRITE);
	
	$arrTables = array(pWIKIDB_tblTables, pWIKIDB_tblRows, pWIKIDB_tblFields);

	foreach ($arrTables as $Table) {
		$Table = $DB->tableName($Table);
		$DB->query("TRUNCATE " . $Table, $fname);
	}
}

//////////////////////// UI HELPERS

function AllowUserToCancel($Delay = 5) {
	global $options;

// If '--quick' wasn't specified, give them 5 seconds to make sure they want to
// proceed with this potentially slow process.
	if(!isset($options['quick'])) {
		print("You can abort by pressing Control-C in the next five seconds...");
		
		$OldVal = "";
		for ($i = $Delay; $i >= 0; $i--) {
			print_c_fixed($OldVal, $i);
			$OldVal = $i;
			sleep(1);
		}
		print("\n");
	}

}

// Replacement for print_c() in maintenance/counter.inc
// Original is buggy when number of characters in $last and $current is different,
// at least on Windows (not tested on *nix).
function print_c_fixed($last, $current) {
// On Windows backspace on its own moves the cursor, but doesn't blank out the
// value.  Therefore we do back-space (move back), space (wipe character) 
// back-space (move back again) instead.
	echo str_repeat( chr(8) . " " . chr(8) , strlen( $last ) ) . $current;
}
